import os
import pytz
from PyQt5.QtWidgets import *
import pandas as pd

import utils
from config import Config
from dataprovider.picarroprovider import PicarroProvider
from gui.ui_datconvert_dialog import Ui_DatconvertDialog
from cfatools.logreader.dataset import DatasetReader


class DatconvertUim:
    def __init__(
        self, picarro_prvd: PicarroProvider, ui: Ui_DatconvertDialog, config: Config
    ):
        self.picarro_prvd = picarro_prvd
        self.ui = ui
        self.config = config
        self.picarro_df = pd.DataFrame()
        self.dataset_name = ""

        self.ui.pushbutton_select_source_files.clicked.connect(self.select_source_files)
        self.ui.pushbutton_select_dest.clicked.connect(self.select_dest_dir)
        self.ui.pushbutton_convert.clicked.connect(self.convert)

    def select_source_files(self):
        # Get the file names of the source .dat file
        if "PYCHARM_HOSTED" in os.environ:
            filenames = QFileDialog.getOpenFileNames(
                caption="Select all the to-be-converted Picarro's .dat files",
                directory=os.path.expanduser("~"),
                filter="DAT (*.dat)",
                options=QFileDialog.DontUseNativeDialog,
            )[0]
        else:
            filenames = QFileDialog.getOpenFileNames(
                caption="Select all the to-be-converted Picarro's .dat files",
                directory=os.path.expanduser("~"),
                filter="DAT (*.dat)"
            )[0]
        if len(filenames) == 0:
            return

        self.ui.listwidget_source_files.addItems(filenames)

        # Read the content of the .dat and convert them as pd.DataFrame
        self.picarro_df = self.picarro_prvd.get_df_from_dat(filenames)

        # Update the date range
        first_date = self.picarro_df.index.min()
        last_date = self.picarro_df.index.max()

        self.ui.datetimeedit_from.setDateTime(first_date)
        self.ui.datetimeedit_from.setMinimumDateTime(first_date)
        self.ui.datetimeedit_from.setMaximumDateTime(last_date)

        self.ui.datetimeedit_to.setDateTime(last_date)
        self.ui.datetimeedit_to.setMinimumDateTime(first_date)
        self.ui.datetimeedit_to.setMaximumDateTime(last_date)

    def select_dest_dir(self):
        directory = utils.get_data_path(self.config, "datasets", "dataset")
        if directory == "":
            return

        self.dataset_name = os.path.split(directory)[1]
        valid, message = DatasetReader.dataset_name_is_valid(self.dataset_name)
        if not valid:
            utils.show_popup(message, "Invalid dataset name!")
            return

        self.ui.lineedit_destdir.setText(directory)

    def convert(self):
        # Filter by datetime range
        first_date = pytz.utc.localize(
            self.ui.datetimeedit_from.dateTime().toPyDateTime()
        )
        last_date = pytz.utc.localize(self.ui.datetimeedit_to.dateTime().toPyDateTime())
        df = self.picarro_df[
            (self.picarro_df.index >= first_date) & (self.picarro_df.index <= last_date)
        ]

        # Build destination filename
        dest_dir = self.ui.lineedit_destdir.text()
        filename = self.dataset_name + "_PICARRO_periodic.log"
        full_filename = os.path.join(dest_dir, filename)

        # Write file
        df.to_csv(
            path_or_buf=full_filename,
            mode="w",
            sep="\t",
            float_format="%.3f",
            date_format=utils.datetime_format,
        )

        msg = QMessageBox()
        msg.setIcon(QMessageBox.Information)
        msg.setWindowTitle("Picarro log file created")
        msg.setText(
            "The picarro log file "
            + filename
            + " has successfully been created in "
            + dest_dir
        )
        msg.exec_()
